#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <ESP8266HTTPClient.h>
#include <WiFiClientSecure.h>
#include <DHT.h>
#include <FS.h>  
#include <Ticker.h>

#define DHTTYPE DHT22
#define DHTPIN  2

Ticker intervalTimer;
boolean flag = true;

uint8_t MAC_array_STA[6];
char MAC_char_STA[18];

ESP8266WebServer server(80);

String file_url = "/baseUrl.txt"; //被读取的文件位置和名称，保存接口链接
String file_timer = "/timer.txt"; //被读取的文件位置和名称，保存接口链接

//http://esp01s.zhoubian168.com/public/index.php/index/Esp01s/saveTemp
String baseUrl = "";
String timer = "";
String url = "";
String MAC_ADDR = "";
String temp = "";
String humi = "";

DHT dht(DHTPIN, DHTTYPE, 11); // 11 works fine for ESP8266
 
float humidity, temp_f, temp_c;  // 从传感器读出的数据
String webString="";     // 显示到WEB页面的文本
unsigned long previousMillis = 0;        // 保存最近一次的读数
const long interval = 2000;              // 最短读取间隔时间2秒

void setup(void)
{
  Serial.begin(115200);  // 设置波特率
  delay(100);

  if (!AutoConfig())  //自动配网功能
  {
      SmartConfig();
  }

  WiFi.macAddress(MAC_array_STA);
  for (int i = 0; i < sizeof(MAC_array_STA); ++i) {
    sprintf(MAC_char_STA, "%s%02x", MAC_char_STA, MAC_array_STA[i]);
  }
  MAC_ADDR = MAC_char_STA;  //char和String的转换，直接赋值就行了
  
  Serial.println("");
  Serial.println("DHT Weather Reading Server");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
  Serial.print("MAC address: ");
  Serial.println(MAC_ADDR);

  server.on("/", handle_root);
  
  server.on("/setBaseUrl", [](){
    server.send(200, "text/html", "<!DOCTYPE html><html lang=\"zh-CN\" class=\"no-js\"><head><meta charset=\"UTF-8\"><title>设置接口链接</title></head><body><form action=\"/update\" method=\"GET\"><input name=\"url\" type=\"text\" style=\"width:600px; height:20px;\" /><input type=\"submit\" value=\"提交\" /></form><br />1，输入接口链接后，将在接口链接后拼接参数： ?mac_addr=MAC地址&temp=温度&humi=湿度<br />2，请接口接收该参数并处理，MAC地址等于设备序列号，用于后端逻辑使用<br />3，暂时不支持https的接口链接，请使用http协议的接口<br />4，保存后设备会重启，大约需要5秒钟</body></html>");
  });

  server.on("/setTimer", [](){
    server.send(200, "text/html", "<!DOCTYPE html><html lang=\"zh-CN\" class=\"no-js\"><head><meta charset=\"UTF-8\"><title>设置间隔时间</title></head><body><form action=\"/updateTime\" method=\"GET\"><input name=\"timer\" type=\"text\" style=\"width:600px; height:20px;\" /><input type=\"submit\" value=\"提交\" /></form><br />请输入正整数，单位是秒，默认5秒钟</body></html>");
  });
  
  server.on("/showDHT", [](){
    gettemperature();           // 读取数据
    webString="<!DOCTYPE html><html lang=\"zh-CN\" class=\"no-js\"><head><meta charset=\"UTF-8\"><title>显示温湿度</title></head><body>温度：" + String((float)temp_c) + "\r\n湿度：" + String((float)humidity) + "</body></html>";
    //webString="温度：" + String((float)temp_c) + "\r\n湿度：" + String((float)humidity);
    server.send(200, "text/html", webString);
    delay(100);
  });

  server.on("/update", [](){        // 处理更新函数，更新链接
    server.send(200, "text/plain", "OK");
    baseUrlUpdate();
  });

  server.on("/updateTime", [](){        // 处理更新函数，更新间隔
    server.send(200, "text/plain", "OK");
    timerUpdate();
  });

  server.on("/reboot", [](){  //重启
    server.send(200, "text/plain", "OK");
    delay(500);
    reboot();
  });

  server.on("/format", [](){  //格式化
    format();
    server.send(200, "text/plain", "OK");
  });
  
  server.on("/showBaseUrl", [](){  //读取接口链接
    webString="<!DOCTYPE html><html lang=\"zh-CN\" class=\"no-js\"><head><meta charset=\"UTF-8\"><title>接口链接</title></head><body>当前接口链接是：" + baseUrl + "</body></html>";
    server.send(200, "text/html", webString);
  });

  server.on("/showTimer", [](){  //读取间隔时间
    webString="<!DOCTYPE html><html lang=\"zh-CN\" class=\"no-js\"><head><meta charset=\"UTF-8\"><title>间隔时间</title></head><body>当前间隔时间是：" + timer + "秒</body></html>";
    server.send(200, "text/html", webString);
  });
  
  server.begin();

  readbaseUrl(); //读取配置
  readTimer(); //读取配置
  if(timer == "") timer = "5";
  intervalTimer.attach(atoi(timer.c_str()), timerCallback); // 数据发送相隔时间，5秒太快，实际使用时需要改一改，比如改到60秒
  dht.begin();           // 初始化传感器

}

void loop(void)
{
  //return;   //程序停止，调试时用
  while (flag)
  {
    sendData();
    flag = false;
  }

  server.handleClient();
} 

// 定时器回调函数
void timerCallback()
{
  flag = true;
}

void handle_root() {
  server.send(200, "text/html", "<!DOCTYPE html><html lang=\"zh-CN\" class=\"no-js\"><head><meta charset=\"UTF-8\"><title>首页</title></head><body><a href=/showDHT>显示温湿度</a><br /><br /><a href=/setBaseUrl>设置接口链接</a><br /><br /><a href=/setTimer>设置上传间隔时间</a><br /><br /><a href=/showBaseUrl>读取接口链接</a><br /><br /><a href=/showTimer>读取间隔时间</a><br /><br /><a href=/format>初始化配置</a><br /><br /><a href=/reboot>重启设备</a></body></html>");
  delay(100);
}

void reboot(){
  Serial.println("重启");
  ESP.restart();
}

void format(){
  Serial.println("格式化");
  SPIFFS.begin();
  SPIFFS.format();    // 格式化SPIFFS
  reboot();
}

void makeBaseUrlFile(String url = ""){
  SPIFFS.begin();
  File dataFile = SPIFFS.open(file_url, "w");
  dataFile.println(url);       
  dataFile.close(); 
  Serial.print("已写baseUrl内容：");
  Serial.println(url);
}

void makeTimerFile(String timeString = ""){
  SPIFFS.begin();
  File dataFile = SPIFFS.open(file_timer, "w");
  dataFile.println(timeString);       
  dataFile.close(); 
  Serial.print("已写timeString内容：");
  Serial.println(timeString);
}

void baseUrlUpdate(){
  String urlString = server.arg("url");  // 接收url链接，然后保存
  Serial.println("=================");
  Serial.println("接收到的内容：" + urlString);
  Serial.println("=================");    
  makeBaseUrlFile(urlString);
  delay(500);
  reboot();
}

void timerUpdate(){
  String timeString = server.arg("timer");  // 接收间隔数据，然后保存
  Serial.println("=================");
  Serial.println("接收到的内容：" + timeString);
  Serial.println("=================");    
  makeTimerFile(timeString);
  delay(500);
  reboot();
}

void readbaseUrl(){
  SPIFFS.begin();
  //确认闪存中是否有file_url文件
  if (SPIFFS.exists(file_url)){
    Serial.println("有url文件");
  } else {
    Serial.println("没有url文件，重建一个");
    makeBaseUrlFile("");
  }

  //建立File对象用于从SPIFFS中读取文件
  File dataFile = SPIFFS.open(file_url, "r"); 

  String tempUrl = "";

  //读取文件内容并且通过串口监视器输出文件信息
  for(int i=0; i<dataFile.size()-2; i++){   //这里少读2字节，是因为文件后面有一个换行
    tempUrl += (char)dataFile.read();
  }
  Serial.println("");
  baseUrl = tempUrl;
  //完成文件读取后关闭文件
  dataFile.close(); 
  Serial.print("读取到的baseUrl链接：");
  Serial.println(baseUrl);
  tempUrl = "";
}

void readTimer(){
  SPIFFS.begin();
  //确认闪存中是否有file_timer文件
  if (SPIFFS.exists(file_timer)){
    Serial.println("有timer文件");
  } else {
    Serial.println("没有timer文件，重建一个");
    makeTimerFile("");
  }

  //建立File对象用于从SPIFFS中读取文件
  File dataFile = SPIFFS.open(file_timer, "r"); 

  String tempTimer = "";

  //读取文件内容并且通过串口监视器输出文件信息
  for(int i=0; i<dataFile.size()-2; i++){
    tempTimer += (char)dataFile.read();
  }
  Serial.println("");
  timer = tempTimer;
  //完成文件读取后关闭文件
  dataFile.close(); 
  Serial.print("读取到的间隔时间：");
  Serial.println(timer);
  tempTimer = "";
}

void gettemperature() {
  // 间隔读数至少2秒，否则可能会出现数据异常
  unsigned long currentMillis = millis();
 
  if(currentMillis - previousMillis >= interval) {
    // 保存最后一次的读取时间
    previousMillis = currentMillis;   

    humidity = dht.readHumidity();          // 读取湿度 %
    //temp_f = dht.readTemperature(true);     // 读取华氏度
    temp_c = dht.readTemperature();     // 读取摄氏度
    
    // 读取失败，显示到控制台文本
    if (isnan(humidity) || isnan(temp_c)) {
      //Serial.println("传感器读取失败");
      return;
    }
  }
}

void sendData(){
  gettemperature();
  temp = String((float)temp_c);
  humi = String((float)humidity);
  if (isnan(humidity) || isnan(temp_c)) {
    //Serial.println("Read False");          //数据读不到就停止，发布时要打开
    //return;
  }
  url = baseUrl + "?mac_addr=" + MAC_ADDR + "&temp=" + temp + "&humi=" + humi;
  Serial.println(url);
  getUrl(url);
  return;
}

void getUrl(String url){
  HTTPClient http;
  http.begin(url);
  int httpCode = http.GET();
  Serial.printf("[HTTP] GET... code: %d\n", httpCode);
  if (httpCode == 200)
  {
    Serial.println("GOOD！");
  }
}

void SmartConfig()
{
  WiFi.mode(WIFI_STA);
  Serial.println("\r\nWait for Smartconfig...");
  WiFi.beginSmartConfig();
  int i = 0;
  while (1)
  {
    Serial.println(i++);
    delay(1000);                   // 1秒钟刷新一下
    if (WiFi.smartConfigDone())
    {
      Serial.println("SmartConfig Success");
      Serial.printf("SSID:%s\r\n", WiFi.SSID().c_str());
      Serial.printf("PSW:%s\r\n", WiFi.psk().c_str());
      break;
    }
  }
}

bool AutoConfig()
{
    WiFi.begin();
    //如果觉得时间太长可改
    for (int i = 0; i < 20; i++)
    {
        int wstatus = WiFi.status();
        if (wstatus == WL_CONNECTED)
        {
            delay(500);
            Serial.println("WIFI SmartConfig Success");
            Serial.printf("SSID:%s", WiFi.SSID().c_str());
            Serial.println("");
            Serial.printf("PSW:%s\r\n", WiFi.psk().c_str());
            Serial.print("LocalIP:");
            Serial.println(WiFi.localIP());
            Serial.print("GateIP:");
            Serial.println(WiFi.gatewayIP());
            return true;
        }
        else
        {
            Serial.print("WIFI AutoConfig Waiting......");
            Serial.print(wstatus);
            Serial.print("-");
            Serial.println(i);
            delay(1000);
        }
    }
    Serial.println("WIFI AutoConfig Faild!" );
    return false;
}
